#!/usr/local/bin/perl -w
#
######################################################################
# This file is part of PICOwatt.
#
# Copyright (c) 2009 Tenrehte Technologies, Inc. ("TENREHTE")
# Copyright (c) 2010 Tenrehte Technologies, Inc. ("TENREHTE")
# Copyright (c) 2013 Tenrehte Technologies, Inc. ("TENREHTE")
#
# $Revision: 5.1 $
#
# ALL RIGHTS RESERVED.
#
# Licensed Materials - All contents of this file and the design
# represented herein are the sole property of TENREHTE.
#
# This software, binaries and associated documentation are owned by TENREHTE and/or its
# licensors, and may be used only as authorized in the license agreement controlling such use.
# No part of this publication may be reproduced, transmitted, or translated in any form or by
# any means, electronic, mechanical, manual, optical, or otherwise, without prior written
# permission of TENREHTE, or as expressly provided by your license agreement. 
# 
# Trademarks
# 
# Tenrehte, Tenrehte Technologies, Tenrehte Technologies, Inc., Smart Wi-Fi,
# PICO, PICOwatt, PICOmed, PICOmesh are registered trademarks or trademarks of TENREHTE
# in the United States and/or other countries. Product and company names mentioned herein
# are trademarks and/or registered trademarks of their respective companies. TENREHTE
# publication of information regarding any third party's products or services does not
# constitute TENREHTE approval, warranty, or endorsement thereof. 
# 
# Important Information
# 
# TENREHTE reserves the right to make changes, updates, or corrections, at any time
# to the information in this document and to its products, or to discontinue any product or
# service at any time without notice. No part of this document may be reproduced or
# transmitted in any form or by any means for any purpose without the express written
# permission of TENREHTE. Nothing in this document shall be construed to grant any license or
# right, either expressly, by implication, estoppel, or otherwise, to any intellectual
# property right of TENREHTE. All information is provided on an "as is" basis, and to the
# extent permitted under applicable law, TENREHTE disclaims all warranties, express or
# implied, including, but not limited to any conditions of quality or any warranties arising
# by law, statute, usage of trade or course of dealing. To the extent permitted by
# applicable law, in no event shall TENREHTE be liable for any loss or damage, however caused
# and on any theory of liability, arising from the use of any information in this document
# or any error or omission in such information. 
# 
# Certain applications using semiconductor products may involve potential risks of death,
# personal injury, or severe property or environmental damage (critical applications).
# TENREHTE products are not designed, authorized, or warranted to be suitable for use in
# life-support devices or systems or other critical applications. Inclusion of TENREHTE
# products in such applications is understood to be fully at the customer's risk. In order
# to minimize risks associated with the customer's applications, adequate design and
# operating safeguards must be provided by the customer to minimize inherent or procedural
# hazards. TENREHTE assumes no liability for applications assistance or customer product
# design. 
# 
# Destination Control Statement
# 
# All technical data contained in this publication is subject to the export control laws of
# the United States of America. Disclosure to nationals of other countries contrary to the
# United States law is prohibited. It is the reader's responsibility to determine the
# applicable regulations and to comply with them. 
# 
# Disclaimer
#
# THE SOFTWARE IS PROVIDED "AS IS" AND TENREHTE DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL TENREHTE BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
######################################################################
#
#

use strict;
use Fcntl;
use POSIX;
use Getopt::Long qw(:config auto_version);
use LWP::Simple;

$| = 1 ;

my $version = '$Revision: 5.1 $' ;
$version =~ s/^\$// ;
$version =~ s/\s+\$$// ;

#
my $DEBUG = 0 ;
my $help = 0 ;
my $picowatt = "" ;
my $realtime = 0 ;
my $kwhmidnight = 0 ;
my $https = 0 ;

#
# Get command line options
#
my $result = GetOptions (
    "picowatt=s"  => \$picowatt ,
    "realtime"  => \$realtime ,
    "kwhmidnight"  => \$kwhmidnight ,
    "https"  => \$https ,
    "debug"  => \$DEBUG ,
    "help"  => \$help ,
) ;

if (($help) || (length($picowatt) < 1)) {
    &my_help() ;
    exit(1) ;
}

if ($realtime) {
	&get_realtime($picowatt) ;
}

if ($kwhmidnight) {
	&get_kwhmidnight($picowatt) ;
}

if (!($realtime || $kwhmidnight)) {
	print STDERR "\nNothing to do...\n\n" ;
	&my_help() ;
	exit(1) ;
}

print STDERR "Finished!\n" if $DEBUG ;
exit 0 ;

# Get Real Time Energy
sub get_realtime ($)
{
  	my ($unit) = @_;

	my $this_function = (caller(0))[3];
	#
	# Fetch Real Time Energy
	#

	# Create a WWW request
	my $prot = $https ? "https://" : "http://" ;
	my $req = get("${prot}${unit}/PICOwatt.cgi?btnG=PICOstatus") ;
	my $atime = localtime ;
	print STDERR "\n$this_function: req = [$req]\n" if $DEBUG ;
	
	# Check is a response was obtained...
	if ($req) {
		if ($req =~ m#<tr>\s+<td\s+.+>\s*<b>\s*Usage:\s*</b>\s+Watts\s+Real\s+Time\s*</td>\s+<td\s+.+>\s*(\d+)\s*</td>\s+</tr># ) {
			print STDOUT "${atime} Unit ${unit} Watts Real Time = $1\n" ;
		} else {
			print STDERR "Bad response from ${unit}\n" ;
		}
	} else {
		print STDERR "No response from ${unit}\n" ;
	}

}

# Get kW-h Since Midnight
sub get_kwhmidnight ($)
{
  	my ($unit) = @_;

	my $this_function = (caller(0))[3];
	#
	# Fetch Real Time Energy
	#

	# Create a WWW request
	my $prot = $https ? "https://" : "http://" ;
	my $req = get("${prot}${unit}/PICOwatt.cgi?btnG=PICOstatus") ;
	my $atime = localtime ;
	print STDERR "\n$this_function: req = [$req]\n" if $DEBUG ;
	
	# Check is a response was obtained...
	if ($req) {
		if ($req =~ m#<tr>\s+<td\s+.+>\s*<b>\s*Usage:\s*</b>\s+kW.h\s+Since\s+Midnight\s*</td>\s+<td\s+.+>\s*(\S+)\s*</td>\s+</tr># ) {
			print STDOUT "${atime} Unit ${unit} kW-h Since Midnight = $1\n" ;
		} else {
			print STDERR "Bad response from ${unit}\n" ;
		}
	} else {
		print STDERR "No response from ${unit}\n" ;
	}

}

# Print Help
sub my_help ($)
{
    print "$0\n\t[--help]\n\t[--debug]\n" ;
    print "\t[--realtime] Fetch Real Time Energy\n";
    print "\t[--kwhmidnight] Fetch kW-h Since Midnight\n";
    print "\t[--https] Use HTTPS\n";
    print "\t--picowatt=<PICOwatt unit FQHN or IP address>\n";
}

1; #return true
